#include <stdio.h>
#include <stdlib.h>
#include <SDL/SDL.h>
#include "types.h"
#include "polygon.h"

/* Draws the polygon described by the point list PointList in colour */
/* Colour with all vertices offset by (X,Y) */
#define DRAW_POLYGON(PointList, Colour, Shape, X, Y)           \
    Polygon.Length = sizeof(PointList) / sizeof(struct Point); \
    Polygon.PointPtr = PointList;                              \
    FillPolygon(&Polygon, Colour, Shape, X, Y);

extern S32 FillPolygon(struct PointListHeader *VertexList, S32 Colour, S32 PolyShape, S32 XOffset, S32 YOffset);                                 
extern U8 InitSDLScreenBuf(void);
extern void redraw(void);
extern void cls(void);

/* simple program to exercise the polygo filling routines */
int main(int argc, char *argv[])
{
    S32 i;
    S32 j;
    input_type UserControls;
    memset(&UserControls,0,sizeof(UserControls));
    poly_type SelectPoly = poly0;

    struct PointListHeader Polygon;
    static struct Point Polygon1[] = 
        {{0,0},{100,150},{320,0},{0,200},{220,50},{320,200}};
    static struct Point Polygon2[] = 
        {{0,0}, {320,0}, {320,200}, {0,200}, {0,0}, {50,50},
         {270,50},{270,150},{50,150},{50,50}};
    static struct Point Polygon3[] = 
        {{0,0},{10,0},{105,185},{260,30},{15,150},{5,150},{5,140},
         {260,5},{300,5},{300,15},{110,200},{100,200},{0,10}};
    static struct Point Polygon4[] = 
        {{0,0},{30,-20},{30,0},{0,20},{-30,0},{-30,-20}};    
    static struct Point Triangle1[] = {{30,0},{15,20},{0,0}};
    static struct Point Triangle2[] = {{30,20},{15,0},{0,20}};
    static struct Point Triangle3[] = {{0,20},{20,10},{0,0}};
    static struct Point Triangle4[] = {{20,20},{20,0},{0,10}};
    
    /* try and get a handle to the screen */
    if(InitSDLScreenBuf() == 1)
    {
        /* cant get handle to screen so exit */
        return(1);
    }
    
    /* Draw first complex polygon */
    DRAW_POLYGON(Polygon1, 3, COMPLEX, 0, 0);  
    SelectPoly++;
    redraw();
    cls();
    
    /* loop round until user exits */
    while(UserControls.Exit == 0)
    {
        /* clear last user input */
        memset(&UserControls,0,sizeof(UserControls));
        
        /* delay getting user input by approx 100ms */
        SDL_Delay(100);
        
        /* then see if user was pressed a key */
        GetUserInput(&UserControls);
        
        /* check to see if user has pressed the up button */
        if(UserControls.Forward == 1)
        {
            switch(SelectPoly)
            {   
                case poly1:
                    DRAW_POLYGON(Polygon2, 5, COMPLEX, 0, 0);    
                    redraw();
                    cls();
                    SelectPoly++;
                break;
                
                case poly2:
                    DRAW_POLYGON(Polygon3, 3, COMPLEX, 0, 0);    
                    redraw();
                    cls();
                    SelectPoly++;
                break;
                
                case poly3:
                    for(i = 0;  i < 5; i++)
                    {    
                        for(j = 0;  j < 8; j++)
                        {
                            DRAW_POLYGON(Polygon4, 3, NONCONVEX, 40+(i*60), 30+(j*20));          
                        }      
                    }
                    redraw();
                    cls();
                    SelectPoly++;
                break;     
                
                default:
                    /* do nothing */
                break;        
            }
        }
    }

    return 0;
}
